////////////////////////////////////////////////////////////////////////
// here you find all the final state classes for processes in the     //
// Standard Model. The default data is supplied  through tables.      //
// The cross section part does not need to be overriden and can be    //
// found in Sigma.cc                                                  //
////////////////////////////////////////////////////////////////////////

#include <SigmaSM.hh>

////////////////////////////////////////////////////////////////////////
// start of CC reaction //
 void SigmaCC::FillVertex(GenVertex* vtx, double material[]){
    HepLorentzVector nu_momentum, lepton_out;
    double log_e, finalstate[2], x, y;
    int flavor;
// first obtain the incoming neutrino: nuin
    GenVertex::particle_iterator nuin = 
        vtx->particles_begin(HepMC::parents);
// get 4-momentum and id
    nu_momentum = (*nuin)->momentum();
    flavor = (*nuin)->pdg_id();
 // flavor of outgoing lepton
    flavor=flavor/abs(flavor)*(abs(flavor)-1);
    log_e = log10(nu_momentum.e());
// make final states of the produced particles
// lepton moves in a different direction
    FinalData->ThrowFinal(log_e,finalstate);
    x = finalstate[0];
    y = finalstate[1];
    
    // checks are necessary in case tables contain buggy numbers
    
    if (y<0)
      y=0; 
    if (y>1)
      y=1; 
    if (x<0)
      x=0; 
    if (x>1)
      x=1; 
    
    lepton_out = (1 - y)*nu_momentum;
    if(abs(flavor) == 13)
       lepton_out =  Muon_Out(nu_momentum, x, y);

    GenParticle* lepton  = new GenParticle(lepton_out,flavor,1);

// 3-momentum of X coincides with that of neutrino
    GenParticle* xhadron = new GenParticle(y*nu_momentum,211,1);

    vtx->add_particle_out(lepton);   // add the result to the vertex
    vtx->add_particle_out(xhadron);
 }

////////////////////////////////////////////////////////////////////////
// This method works only for muons.
// It takes the the scattering angle into account
 HepLorentzVector SigmaCC::Muon_Out(HepLorentzVector nu_momentum, 
          double x, double y) {
    HepLorentzVector muout;
    HepVector3D axil, mu_trans, mu_long, nu_p;
    HepDouble nu_px, nu_pz, phi, mu_e, diffr,
              cos_theta, sin_theta;
// primary neutrino 3-momentum and x,z-components
    nu_p  = nu_momentum.vect();
    nu_px = nu_momentum.px();
    nu_pz = nu_momentum.pz();
// Build an arbitrary HepVector3D axil, perpendicular
// to the neutrino 3-momentum and ssume it is {1,0,z};
// as axil*nu_p = 0, px*1+py*0+pz*z=0, => z = -px/pz
    if (nu_pz != 0.0){                // check nu_pz != 0
      axil.set(1.0, 0.0, -nu_px/nu_pz);
      axil = axil.unit();             // unititarization
    }
    else
      axil.set(1.0,0.0,0.0);

    phi  = 2.0*PI*drand48();          // throw an arbitrary azimuth angle
    axil.rotate(phi,nu_p);            // the rotated transversal unitary vector
    mu_e = (1.0-y)*nu_momentum.e();   // energy of outgoing muon
// Find the scattering angle
    diffr = x*y*m_N/mu_e;
    if (diffr<0 || diffr>2) {
      cerr<<"Muon_Out warning: changed (mu_e,x,y)=("<<mu_e<<","<<x<<","<<y;
      y=2./(2.+x*m_N/nu_momentum.e())-1e-10;
      mu_e = (1.0-y)*nu_momentum.e();
      diffr = x*y*m_N/mu_e;
      cerr<<") to ("<<mu_e<<","<<x<<","<<y<<")"<<endl;
    }

    cos_theta = 1 - diffr;
    sin_theta = sqrt(diffr)*sqrt(2 - diffr);

    mu_trans = mu_e*sin_theta*axil;          // transversal  component of muon 3-momentum
    mu_long  = mu_e*cos_theta*nu_p.unit();   // longitudinal component of muon 3-momentum
    muout.setE(mu_e);
    muout.setVect(mu_long+mu_trans);
    return muout;
 }

////////////////////////////////////////////////////////////////////////
// start of NC reaction //
 void SigmaNC::FillVertex(GenVertex* vtx, double material[]){
// first obtain the incoming neutrino: nuin
    GenVertex::particle_iterator nuin =
	vtx->particles_begin(HepMC::parents);
// now get energy and id
    HepLorentzVector nu_momentum = (*nuin)->momentum();
    int    flavor = (*nuin)->pdg_id();
    double  log_e = log10(nu_momentum.e());
// now create the final state
    double finalstate[2];
    FinalData->ThrowFinal(log_e,finalstate);
//    double x = finalstate[0];
    double y = finalstate[1];

// make the final state particles
    GenParticle*  lepton = new GenParticle((1 - y)*nu_momentum,flavor,1);
    GenParticle* xhadron = new GenParticle(y*nu_momentum,211,1); 
    vtx->add_particle_out(lepton);   // add the result to the vertex
    vtx->add_particle_out(xhadron);
 }

//////////////////////////////////////////
//       Glashow Resonance Section      //
//////////////////////////////////////////
 void SigmaGR::FillVertex(GenVertex* vtx, double material[]){
// first obtain the incoming neutrino: nuin
    GenVertex::particle_iterator nuin =
	vtx->particles_begin(HepMC::parents);
// now get energy and id
    HepLorentzVector nu_momentum = (*nuin)->momentum();
// int flavor = (*nuin)->pdg_id();
// double log_e = log10(nu_momentum.e());
// now create the final state
    double finalstate[7]={0.,0.,0.,0.,0.,0.,0.}; // init. final state kinematic
    fill_resonance_state(finalstate);
// make the final state particles
    if        (finalstate[0] > 0) {
      GenParticle* nuelec  = new GenParticle(finalstate[0]*nu_momentum,-12,1);
      GenParticle*   elec  = new GenParticle(finalstate[1]*nu_momentum, 11,1);
      vtx->add_particle_out(nuelec);
      vtx->add_particle_out(elec);
    } else if (finalstate[2] > 0) {
      GenParticle*  numuon = new GenParticle(finalstate[2]*nu_momentum,-14,1);
      GenParticle*    muon = new GenParticle(finalstate[3]*nu_momentum, 13,1);
      vtx->add_particle_out(numuon);
      vtx->add_particle_out(muon);
    } else if (finalstate[4] > 0) {
      GenParticle*   nutau = new GenParticle(finalstate[4]*nu_momentum,-16,1);
      GenParticle*     tau = new GenParticle(finalstate[5]*nu_momentum, 15,1);
      vtx->add_particle_out(nutau);
      vtx->add_particle_out(tau);
    } else {
      GenParticle* xhadron = new GenParticle(finalstate[6]*nu_momentum,211,1);
    vtx->add_particle_out(xhadron);
    }
 }

////////////////////////////////////////////////////////////////////////
 double SigmaGR::GetSigma(const GenParticle* nuin, double material[]) {
   double energy, sigma, SW;
   const double crs0 = GF2*MW2/PI*GeV2_mbarn; // Standard cross-section in mbarn
   energy = nuin->momentum().e();
   SW = 2*m_e*energy/MW2;
// The total width is devided by the lepton (here muon) one 1/RWmu
   sigma = crs0*SW/( (1 - SW)*(1 - SW) + GW2)/RWmu/3; 
   return sigma;
 }

////////////////////////////////////////////////////////////////////////
 void SigmaGR::InitSigma(char *newtoken, char *sigmafile, char *finalfile){
  // no tables are read !
     Init=1;
 }

////////////////////////////////////////////////////////////////////////
// This function fills the final state of W-boson decay:
// It depends on 2 random numbers and returns the array of 7 energy fractions
// Difinition of the array coloumns:
// 0) anu_e; 1) nu_e; 2) anu_mu; 3) nu_mu; 4) nu_tau; 5) tau; 6) hadrons;

 void SigmaGR::fill_resonance_state(double *fstate){
    int j, ichan;
    double y, y1;
    ichan =  ichan_choice();         // channal of decay

// throw the fraction of the transferred energy
    double  rndm_y = drand48();
// fraction of energy transferred to the final antineutrino
    y1 = pow(rndm_y, 1./3);            
// fraction of energy transferred to the final lepton (e.g. $\mu^-$)
    y  = 1 - y1;                   
    for (j=0; j < 7; j++)
      fstate[j] = 0;
// Choose the decay channal according to ichan
    switch (ichan) {
      case 1:  // electron
        fstate[0] = y1; fstate[1] = y;  break;
      case 2:  // muon
        fstate[2] = y1; fstate[3] = y;  break;
      case 3:  // tau
        fstate[4] = y1; fstate[5] = y;  break;
      case 4:  // hadrons
        fstate[6] =  1;                 break;
      default:
        cerr << " Something is wrong with branching ratios" << 
                "  of the resonance W-decay" << endl;
    }
 }

// Choose the W-boson decay channal
 int SigmaGR::ichan_choice(){
// Branching ratios RWe, RWmu, RWtau are defined in AnisGlobal.hh
// decay channel initialized by 0
     int ichan = 0;
     double p1, p2, p3;
     p1 = RWe;
     p2 = p1 + RWmu;
     p3 = p2 + RWtau;
     double rndm = drand48();
     if      ( rndm <= p1 ) ichan = 1;  // electron
     else if ( rndm <= p2 ) ichan = 2;  // muon
     else if ( rndm <= p3 ) ichan = 3;  // tau
     else if ( rndm <=  1 ) ichan = 4;  // hadrons
     else 
      cerr << " Something is wrong with ichan assignment !!! " << endl;
     return ichan;
 }
//////////////// End of the Glashow Resonance ////////////////////////////


